; Variable Manager v.99
; Keith Batten kbatten@gmail.com
; 16 December 1998
;
; Edited with Assembly Studio 86 v3.0
; Tab width set to 4

_AHL_PLUS_2_PG3		equ		4c3fh
_CXCURAPP			equ		0C1B4h

;*********************************************
;* 01 for the home screen                    *
;* 02 for the polynomial solver              *
;* 03 for the simultaneous equation solver   *
;* 05 for the constant editor                *
;* 06 for the vector editor                  *
;* 07 for the matrix editor                  *
;* 08 for the program editor                 *
;* 0c for the interpolate/extrapolate editor *
;* 0e for the list/stat editor               *
;* 12 for tolerance                          *
;* 19 for the table                          *
;* 1a for table setup                        *
;* 1b for link                               *
;* 1d for reset mem prompt                   *
;* 1e for reset defaults prompt              *
;* 1f for reset all prompt                   *
;* 20 for RAM                                *
;* 21 for mode/self test                     *
;* 23 for delete variables                   *
;* 49 for the function editor                *
;* 4a for the window editor                  *
;* 4c for the graph                          *
;* 53 initial conditions                     *
;* 54 axes something                         *
;* 62 for format                             *
;* 96 for cat/var                            *
;* 98 for an error message                   *
;*********************************************

#include	"asm86.h"
#include	"ti86asm.inc"
#include	"ti86abs.inc"
#include	"ram86.inc"

;------BEGIN (SQRT)KEY LOADER------

NameLength	equ 20 + 11
#DEFINE		NAME	.db "Var Manager v.99 KWB"

.org _asm_exec_ram

	nop
	jp		Start
	.dw		0000h
	.dw		Description
Description:
	.db		"(Un)install "
	NAME
	.db		0,0,0

Start:
	ld		hl,key_op_info		; load (sqrt)key op info into OP1
	rst		20h
	call	_FINDSYM			; is there already a (sqrt)key?
	jr		c, Install			; no so install ours

Uninstall:						; there already was a (sqrt)key
	call	_delvar				; so we uninstall it
	res		6,(iy+$24)			; reset the flag for (sqrt)key
	ld		hl, uninstalled
	call	_puts				; tell the user it was uninstalled
	call	_newline
	ret							; done for now

Install:
	ld		hl, installed		; tell the user we are installing it
	call	_puts
	call	_newline

	ld		hl, key_op_info		; load (sqrt)key op info into OP1
	rst		20h
	ld		hl, code_end - code_start	; get the length of the (sqrt)key program
	call	_CREATEPROG			; create the (sqrt)key program
	ld		a, b                ; convert bde to ahl
	ex		de, hl
	call	$4c3f               ; add 2 to ahl to point it to program data
	call	_SET_ABS_DEST_ADDR  ; set the destination to be our new (sqrt)key
	xor		a
	ld		hl,code_end - code_start  ; set number of bytes to copy
	call	_SET_MM_NUM_BYTES   ; as length of program
	xor		a
	ld		hl, code            ; point to the beginning of the program
	call	_SET_ABS_SRC_ADDR
	call	_mm_ldir            ; copy the program code
	set		6,(iy+$24)          ; set the flag for (sqrt)key

	ret

key_op_info:
	.db		$12,$4,$10
	.db		"KEY",0

installed:
	NAME
	.db		" Installed",0

uninstalled:
	NAME
	.db		" Uninstalled",0

;------END (SQRT)KEY LOADER------


;------BEGIN (SQRT)KEY PROGRAM------

code:
.org _asm_exec_ram-2			; set the start of code to the proper place
								; so calls and jps can be executed properly
code_start:
	.db		$8e,$28				; token for an asm program

	nop
	jp		Key_Start
	.dw		0000h
	.dw		Key_Description
Key_Description:
	NAME
	.db		" (DO NOT RUN!)",0
Key_Start:

	call	$479f				; pop OP1

	ld		a, ($d625)			; load a with the last key press
	push	af					; save the key press
	ld		a, (_CXCURAPP)		; load a with the current app state

check_selftest:
	cp		$21
	jr		nz, check_reset_mem

	pop		af					; time to intercept key presses

	cp		$9					; enter
	jp		nz, skip_key

	ld		hl, name			; clear password fields
	ld		b, 18
	call	$437b				; _ldhlz

	ld		hl, pass_op_info	; check and see if we have a password
	rst		20h					; variable
	call	_FINDSYM
	ld		a, $9
	jp		c, skip_key			; no so continue normally

	call	unlock_first		; check the password

	cp		0					; oops wrong password, no soup for you
	ld		a, 1
	jp		z, skip_key
	ld		a, $9
	jp		skip_key			; got it, so selftest

check_reset_mem:
	cp		$1D
	jr		nz, check_reset_defaults
	jr		reset_menu
check_reset_defaults:
	cp		$1E
	jr		nz, check_reset_all
	jr		reset_menu
check_reset_all:
	cp		$1F
	jr		nz, check_delete
reset_menu:
	pop		af					; time to intercept key presses

	cp		$32					; f-4
	jp		nz, skip_key

	ld		hl, name			; clear password fields
	ld		b, 18
	call	$437b				; _ldhlz

	ld		hl, pass_op_info	; check and see if we have a password
	rst		20h					; variable
	call	_FINDSYM
	ld		a, $32
	jp		c, skip_key			; no so continue normally

	call	unlock_first		; check the password

	cp		0					; oops wrong password, no soup for you
	ld		a, $31
	jr		z, skip_key
	ld		a, $32
	jr		skip_key			; got it, so reset

 
check_delete:
	cp		$23					; check we are in the delete mode
	jr		nz, skip_and_pop	; if not make sure to POP a

	ld		hl, $C1EE			; this address holds the current variable
	rst		20h					; name selected, load it into OP1
	call	_FINDSYM			; see if it exists
	jr		c, skip_and_pop		; no so quit


	ld		a, ($FC00 + $370)	; check specific pixels right above the menu bar
	or		a					; to see if a sub menu is active, if it is then
	jp		nz, skip_and_pop	; quit without writing our text


	ld		hl, $3A36			; load hl with coordinates for the third
	ld		(_penCol), hl		; menu item (centered for certain text)
	ld		hl, rename_str		; place our text here
	call	_vputs				; this is a simple but effective patch

	ld		hl, $3A4F			; load hl with coordinates for the fourth
	ld		(_penCol), hl		; menu item (centered for certain text)
	ld		hl, pass_op_info	; check and see if we have a password
	rst		20h					; variable, if so then display correct
	call	_FINDSYM			; text
	jr		nc, unlock_text		; no so give option to lock
lock_text:
	ld		hl, lock_str
	jr		disp_lock_text
unlock_text:
	ld		hl, unlock_str		; unlock text
disp_lock_text:
	call	_vputs				; this is a simple but effective patch

	ld		hl, $C1EE			; this address holds the current variable
	rst		20h					; name selected, load it into OP1
	call	_FINDSYM			; get the variable info

	ld		de, 4
	sbc		hl, de
	ld		a, (hl)				; get its protected/unprotected state
	and		%00000001			; mask out unused bits

	ld		hl, $3A68			; set the coords for fifth menu item
	ld		(_penCol), hl

	ld		(protected), a
	cp		0
	jr		nz, disp_unprot		; its protected so diplay correct text
disp_prot:
	ld		hl, prot_str
	jr		disp_prot_text
disp_unprot:
	ld		hl, unprot_str
disp_prot_text:
	call	_vputs

	pop		af					; time to intercept key presses

	cp		$9					; enter
	jr		z, check_stats
	cp		$33					; f-3
	jr		z, check_stats
	cp		$32					; f-4
	jr		z, check_stats
	cp		$31					; f-5
	jr		z, check_stats

skip_key:
	cp		a					; clear flags so the OS doesn't try
	ret							; to handle the key after we quit

skip_and_pop:
	pop		af					; make sure we have popped a if needed
	jr		skip_key

check_stats:
	ld		(key), a			; save the key pressed
	ld		hl, (_curRow)		; save the cursor row and column for
	push	hl					; the OS's use in refreshing the text

	ld		hl, $C1EE			; this address holds the current variable
	rst		20h					; name selected, load it into OP1
	call	_FINDSYM			; get the variable info

	ld		a, (hl)				; store the variable type into type
	ld		(type), a
	push	hl					; save the address of the variable symbol

	ld		a, (key)			; check for this first because we want this
	cp		$32					; always to be availible even when the others
	jp		z, lock_unlock		; are locked out

	ld		hl, pass_op_info	; check and see if we have a password
	rst		20h					; variable, if so then lock out functions
	call	_FINDSYM
	jr		nc, locked			; yes so dont allow use of delete and rename

	ld		a, (key)			; check this next so we can individually lock
	cp		$31					; out the following functions
	jp		z, protect_unprotect

	ld		a, (protected)		; check if this specific variable is protected
	cp		0
	jr		nz, locked			; yes so lock out functions
	
	ld		a, (key)			; everything is fine so allow renaming
	cp		$33					; and deleting
	jp		z, rename
	pop		hl
	pop		hl
	cp		a
	ret

locked:
	pop		hl					; function is locked so we dont want any
	pop		hl					; keypresses going through
	xor		a
	cp		a
	ret

rename:
	ld		hl, (_curRow)		; increment the cursor column
	inc		h					; so we are at the start of the
	ld		(_curRow), hl		; variable name

	ld		hl, blank			; clear the variable name from the screen
	call	_puts

	ld		hl, (_curRow)
	ld		h, 1				; reset the column to the start of
	ld		(_curRow), hl		; the variable name

	xor		a
	ld		(case), a			; reset the case flag we will be using
	ld		(size), a			; reset the variable length

	ld		a, $E1
 	call	_putmap
rename_loop:
    ld      a, (case)			; get the current case we are in
    add     a, $E1				; add to above to get correct cursor
    call    _putmap				; display it

	ld		a, (size)			; check to make sure the size doesn't
	cp		8					; get longer than 8 characters, if it
	jp		z, putname			; does then end input and save it

	call	set_case			; load the address of the correct keymap
								; into hl and the correct letter offset
								; into b

	push	bc
	push	hl					; save the address
	call	_getky				; get a key
	pop		hl					; restore address
	pop		bc

	ld		e, a				; add the key value to the keymap address
	ld		d, 0				; to get the value of the key pressed
	add		hl, de
	ld		a, (hl)
	cp		0					; no key
	jr		z, rename_loop		; restart the loop
	cp		1					; enter
	jp		z, putname			; save the new name
	cp		2					; alpha
	jr		z, alpha			; swap between alpha modes
	cp		3					; second
	jr		z, second			; change to second mode
	cp		4					; exit
	jr		z, noname			; quit without renaming
	cp		5					; del
	jr		z, backspace		; go back one character if possible
	add		a, b				; no special key was pressed so display
	call	_putc				; the correct character

	ld		b, a				; save the character in b
	ld		hl, name			; load the offset of the name into hl
	ld		a, (size)			; load the size of name into a
	ld		e, a				; load de with a
	ld		d, 0				; add the offset of the name with the
	ld		a, b				; name length to get the offset to
	add 	hl, de				; put the character in
	ld		(hl), a				; load new character

	ld		a, (size)			; increment the size by one
	inc		a
	ld		(size), a

	ld		a, (case)			; display the correct cursor
	add		a, $E1
	call	_putmap
	jr		rename_loop			; restart the loop

set_case:						; get the offset for the correct keymap
								; and the value to add to the char
	ld		a, (case)			; check what mode we are in
	cp		2					; are we in second mode
	jr		z, set_case_second	; yes so get the second key map into hl
	ld		hl, alpha_keymap	; no so get the alpha key map into hl
	ld		b, 0				; reset the added value
	cp		0					; we are in ALPHA mode
	ret		z					; so return
	ld		b, 32				; we are in alpha mode so we need to add 32
	ret							; to the character to get the lower case char
set_case_second:
	ld		hl, second_keymap	; load hl with the second keymap offset
    ld		b, 0				; reset the added value
	ret

alpha:
	ld		hl, case			; load the current case into a
	ld		a, (hl)
	cp		0					; are we in upper case mode now?
	jr		z, alpha_lower		; yes, so set it to lower case mode
	ld		(hl), 0				; no, so set it to upper case mode
	jr		rename_loop			; restart the loop
alpha_lower:
	ld		(hl), 1				; set case to lower case
	jp		rename_loop			; restart the loop
	
second:
	ld		a, 2                ; set the case to second mode
	ld		(case), a
	jp		rename_loop         ; restart the loop

backspace:						; erase the last character if possible
	ld		a, (size)			; make sure the size is greater than zero
	cp		0
	jp		z, rename_loop		; nope, so dont backspace, restart the loop
	dec		a					; yup so decrease size by one
	ld		(size), a			; set the size
	ld		hl, (_curRow)		; write a ' ' over the last character
	ld		a, ' '
	call	_putmap
	dec		h					; decrement the column
	ld		(_curRow), hl
	jp		rename_loop			; restart the loop

noname:							; dont save the name
	pop     hl					; pop the address of the variable symbol
	pop     hl					; pop the row, col coords
	ld      a, $2E				; refresh the screen on quit
	cp		a
	ret

putname:						; now for the hard part, we have to play some
								; tricks with the vat to change the size of
								; the name, we will have to write the
								; function to handle increases, decreases and
								; static sizes. the way i went about this is
								; to first delete the name totally from the
								; vat (shrink the vat by the length of the
								; var name.  then i increase it by the length
								; of the new var name. that way it always
								; runs through the same functions

	ld		a, (size)			; get the size of the new variable name
	cp		0					; its zero so we want to quit without save
	jr		z, noname

	pop		hl					; pop the address of the variable's symbol
	ld		d, 0				; subtract 5 to it to get the start of the
	ld		e, 5				; variable name since the vat is backwards
	sbc		hl, de
	push	hl					; save the new address
	ld		(sizeaddr), hl		; save this address in sizeaddr

	ld		a, (size)			; get the size of the new variable name
	ld		b, a				; save that in b
	ld		c, a				; and in c
	ld      a, (hl)				; load the length of the old variable name
	cp		b					; compare the two lengths
	pop		hl					; decrease the address to point to the start
	dec		hl					; of the variable, again because the vat is
	push	hl					; backwards, and then resave it
	jr		z, writename		; the lengths were the same so we dont have
								; to start moving things around, we can just
								; overwrite the old name

; hl holds address of beginning of name
; sizeaddr holds address of size
; a holds old name size
; c holds new name size

	cp		a					; clear flags
	ld		b, a				; save the old length into b
	ld		a, c
	ld		(newlength), a		; save the new length into newlength
	ld		a, b				; get the old length back to a

	pop		hl					; get start of name
	push	hl
	ld		d, 0
	ld		e, a				; get length of name
	sbc		hl, de				; subtract them
	push	hl					; hl = start of name - length of name

	ld		de, (VAT_END)		; get end of VAT
	sbc		hl, de				; we already have (start of name - length of name) in hl
	ld		b, h
	ld		c, l				; bc = (start of name - length of name) - end of VAT

	pop		hl					; restore the proper values
	pop		de
	push	de

	lddr						; move vat to overwrite old name

	
	ld		b, a				; save the old length in b
	ld		a, (newlength)		; save the new length in c
	ld		c, a

	ld		a, (size)			; load the length of the new size into a
	ld		hl, (VAT_END)		; get the address for the end of the vat
	ld		d, 0				; add the length of the old name to it
	ld		e, b				; since we have moved it to get the true
	add		hl, de				; end of the vat
	ld		b, 0				; now set hl to point to the end of the
	sbc		hl, bc				; vat after we insert our new name
	ld		(VAT_END), hl		; save that new address in 2 places (???)
	ld		(VAT_END + 3), hl	; not completely sure about this one

	ld		hl, (sizeaddr)		; put the new length into sizeaddr 
	ld		(hl), a

	pop		de					; load de with the place we want to insert
	push	de					; the new name into
	ld		hl, (VAT_END)		; get the end of the vat
	ex		de, hl				; swap hl and de for the subtraction
	sbc		hl, de				; subtract to find the number of bytes
	ld		b, h				; we want to move and put that in bc
	ld		c, l
	inc		bc					; increment bc so it points to the right spot

	ld		hl, (VAT_END)		; get the address for the end of the vat
	ld		d, 0				; in hl, then subtract the length of the new
	ld		e, a				; name to get the address for the new end
	sbc		hl, de				; of the vat
	ld		d, h				; save that into de
	ld		e, l				; get the current end of the vat into hl
	ld		hl, (VAT_END)
	ldir						; move the vat back to make room for the name

    ld      hl, (sizeaddr)		; get the address for the length byte
	dec		hl					; subtract one the get the start of the name
	ld		a, (size)			; get the new size
	ld		c, a				; save it in both b and c
	ld		b, a

;hl holds address of beginning of name
;b holds size
;c holds size
writename:						; now we actually copy the name
	ld		hl, name			; get the address for the new name into hl
	ld		d, 0
	ld		a, c				; get the offset for byte to read
	sub		b					; by subtracting the bytes left from the
								; total bytes

	ld		e, a				; add that offset to hl to get the address
	add		hl, de				; for the next byte to read
	ld		a, (hl)				; load that byte into a

	pop		hl					; get the address for the new name
	push	hl
	ld		(hl), a				; put the character in there
	dec		b					; decrement the number of chars left to copy
	ld		a, b				; see if we are done
	cp		0
	pop		hl					; decrement the address to write name to
	dec		hl					; and save it
	push	hl
	jr		nz, writename		; we're not done so keep copying

	pop		hl					; pop the address of the variable symbol
	pop		hl					; pop the row, col coords
	ld		a, $2E				; refresh the screen
	cp		a
	ret							; quit

unlock_first:
	ld		a, b                ; convert bde to ahl
	ex		de, hl
	call	$4c3f               ; add 2 to ahl to point it to string data
	call	_SET_ABS_SRC_ADDR   ; set the source to our password string
	xor		a
	ld		hl, 8				; set number of bytes to copy
	call	_SET_MM_NUM_BYTES
	xor		a
	ld		hl, name            ; set the distination as name
	call	_SET_ABS_DEST_ADDR
	call	_mm_ldir            ; copy the data

	ld		hl, name2
	ld		(type), hl
	call	get_password		; enter correct password to procede
	call	compare_password
	ld		hl, $0D00
	ld		(_curRow), hl
	ld		hl, blank
	call	_puts
	ret

lock_unlock:
	ld		hl, name			; clear password fields
	ld		b, 18
	call	$437b				; _ldhlz

	ld		hl, pass_op_info	; check and see if we have a password
	rst		20h					; variable
	call	_FINDSYM
	jr		c, lock				; no so lock
unlock:
	call	unlock_first
	cp		0					; oops wrong password, no soup for you
	jr		z, lock_next
	ld		hl, pass_op_info	; get the stuff for the password
	rst		20h					; variable
	call	_FINDSYM
	call	_delvar				; remove password

	ld		hl, locked_vmanage_op_info
	rst		20h					; unhide VManage variable
	call	_FINDSYM
	jr		c, lock_next		; oops not found, we'll have to quit

	ld		(hl), $12			; change it to a program
	ld		e, 6				; subtract 6 to it to get the start of the
	ld		d, 0				; variable name since the vat is backwards
	sbc		hl, de
	ld		(hl), 'V'			; change the first character back to 'V'

	jr		lock_next

lock:
	ld		hl, name
	ld		(type), hl
	call	get_password		; get a password to lock the functions
	ld		hl, $0D00
	ld		(_curRow), hl
	ld		hl, blank
	call	_puts

	ld		hl, pass_op_info	; load password variable name into OP1
	rst		20h
	ld		hl, 8				; make it 9 bytes long
	call	_CREATESTRNG		; create a password string

	ld		a, b                ; convert bde to ahl
	ex		de, hl
	call	$4c3f               ; add 2 to ahl to point it to string data
	call	_SET_ABS_DEST_ADDR  ; set the destination to be our new string
	xor		a
	ld		hl, 8				; set number of bytes to copy
	call	_SET_MM_NUM_BYTES
	xor		a
	ld		hl, name            ; point to the beginning of the name
	call	_SET_ABS_SRC_ADDR
	call	_mm_ldir            ; copy the data

	ld		hl, vmanage_op_info
	rst		20h					; hide VManage variable
	call	_FINDSYM
	jr		c, lock_next		; oops not found, we'll have to quit

	ld		(hl), $0C			; change it to a string so it won't appear
								; in a shell
	ld		e, 6				; subtract 6 to it to get the start of the
	ld		d, 0				; variable name since the vat is backwards
	sbc		hl, de
	ld		(hl), 1				; hide it by changing the first character
	
lock_next:
	pop		hl
	pop		hl
	ld		a, $2E				; refresh the screen
	cp		a
	ret							; quit

compare_password:
	ld		hl, 0
	ld		(size), hl
compare_password_loop:
	ld		de, name
	ld		hl, (size)
	add		hl, de
	ld		a, (hl)
	ld		b, a
	ld		de, name2
	ld		hl, (size)
	add		hl, de
	ld		a, (hl)
	cp		b
	ld		a, 0
	ret		nz
	ld		a, (size)
	inc		a
	ld		(size), a
	cp		8
	ld		a, 1
	ret		z
	jr		compare_password_loop

get_password:
	ld		hl, $0D00
	ld		(_curRow), hl
	ld		hl, line
	call	_puts
	ld		hl, $0D00
	ld		(_curRow), hl
	ld		hl, 0
	ld		(size), hl
get_password_loop:
	call	_getky
	cp		0
	jr		z, get_password_loop

	cp		9
	ret		z

	ld		de, (type)
	ld		hl, (size)
	add		hl, de

	ld		(hl), a
	ld		a, '*'
	call	_putc
	ld		a, (size)
	inc		a
	ld		(size), a
	cp		8
	ret		z
	jr		get_password_loop

protect_unprotect:
	ld		hl, $C1EE			; this address holds the current variable
	rst		20h					; name selected, load it into OP1
	call	_FINDSYM			; get the variable info
	ld		de, 4
	sbc		hl, de
	ld		a, (hl)				; check whether it is currently protected or not
	xor		%00000001			; flip that last bit
	ld		(hl), a	
	pop		hl					; pop everything before we quit
	pop		hl
	xor		a
	cp		a
	ret							; quit


alpha_keymap:
	.db		$00,$00,$00,$00,$00,$00,$00,$00,$00
	.db		$01,'X','T','O','J','E',$00,$00
	.db		$00,'W','S','N','I','D',$00,$00
	.db		'Z','V','R','M','H','C',$00,$05
	.db		'Y','U','Q','L','G','B',$00,$00
	.db		$00,$00,'P','K','F','A',$00,$02
	.db		$00,$00,$00,$00,$00,$03,$04,$00

second_keymap:
	.db		$00,$00,$00,$00,$00,$00,$00,$00,$00
	.db		$01,$00,$00,$00,$00,$00,$00,$00
	.db		$D2,'3','6','9',$00,$00,$00,$00
	.db		'.','2','5','8',$00,$00,$00,$05
	.db		'0','1','4','7',$00,$00,$00,$00
	.db		$00,$00,$00,$00,$00,$00,$00,$02
	.db		$00,$00,$00,$00,$00,$03,$04,$00

size:
	.db		0
case:
	.db		0
newlength:                      ; multiple labels to make the code a little
key:                            ; bit easier to read
	.db		0

type:                           ; multiple labels to make the code a little
sizeaddr:                       ; bit easier to read
	.db		0,0

blank:
	.db		"        ",0
line:
	.db		"--------",0
name:
	.db		0,0,0,0,0,0,0,0,0
name2:
	.db		0,0,0,0,0,0,0,0,0

strng:
	.db		"STRNG",0
prgm
	.db		"PRGM ",0
rename_str:
	.db		"Name",0
lock_str:
	.db		"  Lock",0
unlock_str:
	.db		"Unlck",0
prot_str:
	.db		"  Prot ",0
unprot_str:
	.db		"UnPrt",0
pass_op_info:
	.db		$0C,$5,$1
	.db		"PASS",0
vmanage_op_info:
	.db		$12,$7
	.db		"VManage",0
locked_vmanage_op_info:
	.db		$0C,$7,$1
	.db		"Manage",0
lockedvars_op_info:
	.db		$0C,$5,$1
	.db		"LOCKED",0
protected:
	.db		0

code_end:


.end

